/**
 *  This file is part of Escudo.
 *
 *  Escudo is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Escudo is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Escudo.  If not, see <http://www.gnu.org/licenses/>.
 */


with (EscudoLib) {
  top.Escudo.Categories =  {
    i_category: "INSERT INTO category(\"category\") VALUES (?1)",
    s_category: "SELECT id FROM category WHERE category=?1",
    s_pattern: "SELECT category FROM category WHERE category LIKE ?1 ORDER BY category",
    
    /****************************************
     * Initialize / Shutdown
     */
    init: function() {
      addObserver("invalid", this.Observer);
      addObserver("refresh", this.Observer);
    },

    Observer : {
      observe: function(subject, topic, data) {
        var tree = document.getElementById("tree-categories");
        var param = document.getElementById("param-categoriesAccount");

        if (topic == "invalid") { param.textContent = data; }
        tree.builder.rebuild();
      }
    },

    /****************************************
     * Helper function
     */
    getCategoryMatchingPattern: function(pattern) {
      let statement = getStatement(this.s_pattern);
      statement.bindStringParameter(0, "%" + pattern + "%");

      var values = [];
      try {
        while (statement.executeStep()) {
          values.push(statement.getString(0));
        }
      } finally {
        statement.reset();
      }
      return values;
    },

    getCategoryId: function(category) {
      let statement = getStatement(this.s_category);
      statement.bindStringParameter(0, category);

      var id = undefined;
      try {
        while (statement.executeStep()) {
          id = statement.getInt32(0);
        }
      } finally {
        statement.reset();
      }
      return id;
    },

    getOrCreateCategory: function(category) {
      var id = this.getCategoryId(category);
      if (id == undefined) { id = this.insert(category); }
      return id;
    },

    insert: function(category) {
      let statement = getStatement(this.i_category);
      statement.bindStringParameter(0, category);
      statement.execute();
      return lastInsertRowID();
    }
  }
  top.Escudo.Categories.init();
}
