/**
 *  This file is part of Escudo.
 *
 *  Escudo is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Escudo is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Escudo.  If not, see <http://www.gnu.org/licenses/>.
 */

with (EscudoLib) {
  top.Escudo.Operations =  {
    d_op: "DELETE FROM operation WHERE id=?1",
    i_op: "INSERT INTO operation (\"expense\", \"income\", \"date\", \"category\", \"payee\", \"account\") VALUES (?1, ?2, ?3, ?4, ?5, ?6)",
    u_op: "UPDATE operation SET date=?1, income=?2, expense=?3 WHERE id=?4",

    
    /****************************************
     * Initialize / Shutdown
     */
    init: function() {
      addObserver("refresh", this.Observer);
      addObserver("invalid", this.Observer);
    },

    /****************************************
     * Observer
     */
    Observer : {
      observe: function(subject, topic, data) {
        let tree = document.getElementById("tree-op");
        let param = document.getElementById("param-opAccount");

        if (topic == "invalid") { param.textContent = data; }
        tree.builder.rebuild();
      }
    },

    /****************************************
     * Database access
     */
    remove: function(id) {
      if (confirm($STR("conf_delLedger")) == true) {
        var value = getTreeCellValue("tree-op", "op-id");

        let statement = getStatement(this.d_op);
        statement.bindInt32Parameter(0, value);

        statement.execute();
        notifyObserver("refresh", null);
      }
    },

    insert: function(date, debit, credit) {
      let statement = getStatement(this.i_op);

      //get the different values
      let date = document.getElementById("datepicker-addDate").value;
      let debit = document.getElementById("textbox-addExpense").value;
      let credit = document.getElementById("textbox-addIncome").value;
      let account = top.Escudo.Accounts.getOrCreateAccount(document.getElementById("texbox-menu").value);
      let category = top.Escudo.Categories.getOrCreateCategory(document.getElementById("texbox-addOp-Category").value);
      let payee = top.Escudo.Payees.getOrCreatePayee(document.getElementById("texbox-addOp-Payee").value);

      //binding
      statement.bindDoubleParameter(0, debit);
      statement.bindDoubleParameter(1, credit);
      statement.bindStringParameter(2, date);
      statement.bindInt32Parameter(3, category);
      statement.bindInt32Parameter(4, payee);
      statement.bindInt32Parameter(5, account);
      statement.execute(); //auto reset

      notifyObserver("refresh", null);
    },

    update: function(id, date, expense, income) {
      let statement = getStatement(this.u_op);

      //get the different values
      let expense = document.getElementById("textbox-updateExpense").value;
      let income = document.getElementById("textbox-updateIncome").value;
      let date = document.getElementById("datepicker-updateDate").value;
      let category = document.getElementById("texbox-updateLedger-Category").value;
      let payee = document.getElementById("texbox-updateLedger-Payee").value;

      //Update the ledger table
      dump("Start binding update ledger: " + this._ledgerEdited + ".\n");
      dump("\tdate: " + date + ".\n");
      dump("\texpense: " + expense + ".\n");
      dump("\tincome: " + income + ".\n");
      statement.bindStringParameter(0, date);
      statement.bindDoubleParameter(1, income);
      statement.bindStringParameter(2, expense);
      statement.bindStringParameter(3, this._ledgerEdited);
      dump("End binding update ledger: " + this._ledgerEdited + ".\n");
      statement.execute(); //auto reset

      //Connect the ledger to the [new] category and payee
      top.Escudo.Categories.connect(this._ledgerEdited, category);
      top.Escudo.Payees.connect(this._ledgerEdited, payee);

      notifyObserver("refresh", null);
    }
  }
  //init
  top.Escudo.Operations.init();
}
